// list~: convert the results of an FFT analysis from an audio signal into a list
// original by: Timothy Place, Cycling '74
// license: MIT

#include "ext.h"			// standard Max include
#include "ext_obex.h"		// required for all modern Max objects
#include "z_dsp.h"			// required for audio objects
#include "r_pfft.h"			// required for accessing pfft~ object info


// struct to represent the object's state
typedef struct _fftlist {
	t_pxobject	ob;				// the object "base class" (t_pxobject for audio objects instead of t_object)
	double		fftsize;		// the size of the FFT we are normalizing (the number of bins in the FFT)
	t_atom*		real;			// the real component of the FFT actively being written into
	t_atom*		imag;			// the imaginary component of the FFT actively being written into
	t_atom*		real_complete;	// the last complete frame of real values
	t_atom*		imag_complete;	// the last imaginary frame of real values
	void*		real_outlet;	// an outlet for the real lists
	void*		imag_outlet;	// an outlet for the imaginary lists
} t_fftlist;


// method prototypes
void *fftlist_new(t_symbol *s, long argc, t_atom *argv);
void fftlist_free(t_fftlist *x);
void fftlist_assist(t_fftlist *x, void *b, long m, long a, char *s);
void fftlist_bang(t_fftlist *x);
void fftlist_dsp64(t_fftlist *x, t_object *dsp64, short *count, double samplerate, long maxvectorsize, long flags);
void fftlist_perform64(t_fftlist *x, t_object *dsp64, double **ins, long numins, double **outs, long numouts, long sampleframes, long flags, void *userparam);


// global class pointer variable
static t_class *fftlist_class = NULL;


//***********************************************************************************************

void ext_main(void *r)
{
	t_class *c = class_new("fft.list~", (method)fftlist_new, (method)fftlist_free, (long)sizeof(t_fftlist), 0L, A_GIMME, 0);

    class_addmethod(c, (method)fftlist_bang,	"bang", 0);
	class_addmethod(c, (method)fftlist_dsp64,	"dsp64",	A_CANT, 0);
	class_addmethod(c, (method)fftlist_assist,	"assist",	A_CANT, 0);

	class_dspinit(c);
	class_register(CLASS_BOX, c);
	fftlist_class = c;
}


void *fftlist_new(t_symbol *s, long argc, t_atom *argv)
{
	t_fftlist *x = (t_fftlist *)object_alloc(fftlist_class);

	if (x) {
		dsp_setup((t_pxobject *)x, 3);	// MSP inlets: arg is # of inlets and is REQUIRED!
        x->ob.z_misc = Z_NO_INPLACE;

		// we create the outlets in right-to-left order and store their pointers in our struct
        x->imag_outlet = outlet_new(x, NULL);
        x->real_outlet = outlet_new(x, NULL);
        
		if (argc > 0) {
			// if there is an argument, use it for the FFT size
			x->fftsize = atom_getfloat(argv);
		}
		else {
			// if there is no argument, check to see if we are inside of a pfft~ patcher
			// if so, use the FFT size from that
			// otherwise fall back on a default value that may or may not be correct.

			t_pfftpub *pfft = (t_pfftpub*)gensym("__pfft~__")->s_thing;
			if (pfft)
				x->fftsize = pfft->x_fftsize;
			else
				x->fftsize = 512;
		}

		// allocate memory for the number of atoms in our lists
        x->real = (t_atom*)sysmem_newptr(sizeof(t_atom) * x->fftsize);
        x->imag = (t_atom*)sysmem_newptr(sizeof(t_atom) * x->fftsize);
        x->real_complete = (t_atom*)sysmem_newptr(sizeof(t_atom) * x->fftsize);
        x->imag_complete = (t_atom*)sysmem_newptr(sizeof(t_atom) * x->fftsize);
	}
	return (x);
}


void fftlist_free(t_fftlist *x)
{
	// dsp_free() must *always* be called first in the free routine of audio objects
    dsp_free((t_pxobject*)x);

	// only after dsp_free() is called will it be safe to free our memory
    sysmem_freeptr(x->real);
    sysmem_freeptr(x->imag);
    sysmem_freeptr(x->real_complete);
    sysmem_freeptr(x->imag_complete);
}


void fftlist_assist(t_fftlist *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_INLET) { //inlet
		sprintf(s, "I am inlet %ld", a);
	}
	else {	// outlet
		sprintf(s, "I am outlet %ld", a);
	}
}


void fftlist_bang(t_fftlist *x)
{
    outlet_list(x->imag_outlet, NULL, x->fftsize, x->imag_complete);
    outlet_list(x->real_outlet, NULL, x->fftsize, x->real_complete);
}


// registers a function for the signal chain in Max
void fftlist_dsp64(t_fftlist *x, t_object *dsp64, short *count, double samplerate, long maxvectorsize, long flags)
{
	object_method(dsp64, gensym("dsp_add64"), x, fftlist_perform64, 0, NULL);
}


// this is the 64-bit perform method audio vectors
void fftlist_perform64(t_fftlist *x, t_object *dsp64, double **ins, long numins, double **outs, long numouts, long sampleframes, long flags, void *userparam)
{
	t_double *in_real = ins[0];
    t_double *in_imag = ins[1];
    t_double *in_index = ins[2];
    int i;
    
    for (i=0; i<sampleframes; i++) {
        double real = in_real[i];
        double imag = in_imag[i];
        int index = CLAMP(in_index[i], 0, x->fftsize-1);
       
        atom_setfloat(x->real+index, real);
        atom_setfloat(x->imag+index, imag);
        
        if (index == x->fftsize-1) {
			// when the frame is complete, then we
			// copy the complete frame so that it can be delivered in response to a 'bang'
			//
			// this protects us from sending out a partial frame, for example:
			// if the FFT size is 512 and the signal vector size is 64, then it will take
			// 8 calls to this processing method before the frame is complete.
			//
			// however, this still isn't actually threadsafe.
			// there is a chance that the main thread could be called in the middle of copying this memory.
			// to address that we could perform this copy using a lock-free queue
			//
			// in this case the threadsafety problem will not result in memory corruption,
			// but only in "torn" frames being reported.
			// thus we can leave the problem to be addressed at a later time.

            sysmem_copyptr(x->real, x->real_complete, sizeof(t_atom) * x->fftsize);
            sysmem_copyptr(x->imag, x->imag_complete, sizeof(t_atom) * x->fftsize);
        }
   }
}
