// fft.normalize~: a simple audio object for normalizing the results of FFT analysis on real audio input
// original by: Timothy Place, Cycling '74
// license: MIT

#include "ext.h"			// standard Max include
#include "ext_obex.h"		// required for all modern Max objects
#include "z_dsp.h"			// required for audio objects
#include "r_pfft.h"			// required for accessing pfft~ object info


// struct to represent the object's state for each instance
typedef struct _normalize {
	t_pxobject	ob;			// the object "base class" (t_pxobject for audio objects instead of t_object)
    double		fftsize;	// the size of the FFT we are normalizing (the number of bins in the FFT)
} t_normalize;


// method prototypes
void *normalize_new(t_symbol *s, long argc, t_atom *argv);
void normalize_free(t_normalize *x);
void normalize_assist(t_normalize *x, void *b, long m, long a, char *s);
void normalize_dsp64(t_normalize *x, t_object *dsp64, short *count, double samplerate, long maxvectorsize, long flags);
void normalize_perform64(t_normalize *x, t_object *dsp64, double **ins, long numins, double **outs, long numouts, long sampleframes, long flags, void *userparam);


// global class pointer variable
static t_class *normalize_class = NULL;


//***********************************************************************************************

void ext_main(void *r)
{
	// object initialization, note the use of dsp_free for the freemethod, which is required
	// unless you need to free allocated memory, in which case you should call dsp_free from
	// your custom free function.

	t_class *c = class_new("fft.normalize~", (method)normalize_new, (method)dsp_free, (long)sizeof(t_normalize), 0L, A_GIMME, 0);

	class_addmethod(c, (method)normalize_dsp64,		"dsp64",	A_CANT, 0);
	class_addmethod(c, (method)normalize_assist,	"assist",	A_CANT, 0);

	class_dspinit(c);
	class_register(CLASS_BOX, c);
	normalize_class = c;
}


void *normalize_new(t_symbol *s, long argc, t_atom *argv)
{
	t_normalize *x = (t_normalize *)object_alloc(normalize_class);

	if (x) {
		dsp_setup((t_pxobject *)x, 2);	// MSP inlets: arg is # of inlets and is REQUIRED!
        x->ob.z_misc = Z_NO_INPLACE;
        
		outlet_new(x, "signal"); 		// signal outlet (note "signal" rather than NULL)
        outlet_new(x, "signal"); 		// signal outlet (note "signal" rather than NULL)

		if (argc > 0) {
			// if there is an argument, use it for the FFT size
            x->fftsize = atom_getfloat(argv);
		}
		else {
			// if there is no argument, check to see if we are inside of a pfft~ patcher
			// if so, use the FFT size from that
			// otherwise fall back on a default value that may or may not be correct.

			t_pfftpub *pfft = (t_pfftpub*)gensym("__pfft~__")->s_thing;
			if (pfft)
				x->fftsize = pfft->x_fftsize;
			else
				x->fftsize = 512;
 		}
	}
	return (x);
}


void normalize_assist(t_normalize *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_INLET) { //inlet
		sprintf(s, "I am inlet %ld", a);
	}
	else {	// outlet
		sprintf(s, "I am outlet %ld", a);
	}
}


// registers a function for the signal chain in Max
void normalize_dsp64(t_normalize *x, t_object *dsp64, short *count, double samplerate, long maxvectorsize, long flags)
{
	object_method(dsp64, gensym("dsp_add64"), x, normalize_perform64, 0, NULL);
}


// this is the 64-bit perform method audio vectors
void normalize_perform64(t_normalize *x, t_object *dsp64, double **ins, long numins, double **outs, long numouts, long sampleframes, long flags, void *userparam)
{
	t_double *in_real = ins[0];		// we get audio for each inlet of the object from the **ins argument
    t_double *in_imag = ins[1];
	t_double *out_real = outs[0];	// we get audio for each outlet of the object from the **outs argument
    t_double *out_imag = outs[1];
	int n = sampleframes;

	// this perform method simply copies the input to the output, offsetting the value
    while (n--) {
        *out_real++ = *in_real++ / (x->fftsize/2.0);
		*out_imag++ = *in_imag++ / (x->fftsize/2.0);
    }
}
